use std::sync::Arc;

use scratchpad::game::{GameError, GameState, TileType};
use tokio::sync::broadcast::Sender;

pub struct GameStateSender {
    game_state: GameState,
    sender: Option<Sender<Arc<[TileType]>>>,
}

impl GameStateSender {
    pub const fn new() -> Self {
        Self {
            game_state: GameState::blank(),
            sender: None,
        }
    }

    pub fn set_sender(&mut self, sender: Sender<Arc<[TileType]>>) {
        self.sender.replace(sender);
    }

    pub fn get_board_values(&self) -> scratchpad::game::BoardValues {
        self.game_state.get_board_values()
    }

    pub fn replace_game_state(&mut self, bytes: &[u8]) -> Result<(), GameError> {
        self.game_state.replace_game_state(bytes)
    }

    pub async fn set_tile(&mut self, row: u32, col: u32, value: TileType) -> Result<(), GameError> {
        self.game_state.set_tile(row, col, value)?;
        let sender = match self.sender.as_ref() {
            Some(s) => s,
            None => return Ok(()),
        };
        let new_state = self
            .game_state
            .get_board_values()
            .iter()
            .flat_map(|tiles| tiles.iter())
            .copied()
            .collect::<Arc<_>>();
        if let Err(err) = sender.send(new_state) {
            log::error!("sending from set_tile: {err}");
        }
        Ok(())
    }
}
