use yew::prelude::*;
use yew_router::hooks;

use crate::components::{PageQuery, Pagination, PostLinks};

const ITEMS_PER_PAGE: usize = 10;

#[function_component]
pub fn PostIndex() -> Html {
    let location = hooks::use_location().unwrap();
    let current_page: usize = location.query::<PageQuery>().map(|it| it.page).unwrap_or(1) as _;
    let posts_count = crate::post_list::POSTS.len();
    let total_pages = if posts_count % ITEMS_PER_PAGE == 0 {
        posts_count / ITEMS_PER_PAGE
    } else {
        1 + (posts_count / ITEMS_PER_PAGE)
    };

    let posts = crate::post_list::POSTS
        .iter()
        .skip((current_page - 1) * ITEMS_PER_PAGE)
        .take(ITEMS_PER_PAGE)
        .collect::<Box<_>>();

    html! {
        <div class="section container">
            <h1 class="title">{ "Posts" }</h1>
            <PostLinks posts={posts}/>
            <Pagination
                page={current_page as u32}
                total_pages={total_pages as u32}
                route_to_page={crate::Route::PostIndex}
            />
        </div>
    }
}
